@@ -92,8 +92,6 @@ class Cell:
         try:
             async with websockets.connect(full_url) as ws:
                 await ws.send(json.dumps(auth_payload))
-                print("Listening to Stream...")
-
                 try:
                     while True:
                         try:
@@ -174,13 +172,13 @@ class Cell:
                 async with session.post(url, json=TX) as response:
                     response.raise_for_status()
                     data = await response.json()
-                    if data["success"] == "activated":
-                        async for operation in self.sync():
-                            label = operation.get("label")
-                            if label == "tx_response":
-                                operation_txID = operation.get("txID")
-                                if operation_txID == txID:
-                                    return operation.get("data")
+                    if data.get("success") == True:
+                                if "json" in data.get("response"):
+                                    return data.get("response").get("json")
+                                elif "html" in data.get("response"):
+                                    return "Info: HTML response available. Please activate TX in browser."
+                                else:
+                                    return "Info: Response received but contains no usable content."
                     else:
                         print(data["success"], data["message"])
 
